// Copyright (c) 2001, 2002 Zope Corporation and Contributors.
//
// All Rights Reserved.
//
// This software is subject to the provisions of the Zope Public License,
// Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
// WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
// FOR A PARTICULAR PURPOSE.

using System;
using System.Collections;
using System.Runtime.InteropServices;

namespace Python.Runtime {


    // This file defines support objects for the various forms of trickery
    // we use to connect Python to the CLR. This is all considered private.


    [Serializable()]
    [AttributeUsage(AttributeTargets.Method | AttributeTargets.Delegate)]
    internal class CallConvCdeclAttribute : Attribute {
	public CallConvCdeclAttribute() {}
    }

    [Serializable()]
    [AttributeUsage(AttributeTargets.Method | AttributeTargets.Delegate)]
    internal class PythonMethodAttribute : Attribute {
	public PythonMethodAttribute() {}
    }


    [StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
    internal class PyObjectHead {
	public IntPtr ob_refcnt;
	public IntPtr ob_type;
	public IntPtr ob_data;
	public IntPtr ob_dict;
    }


    [StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
    internal class PyTypeObject {
	public IntPtr ob_refcnt = IntPtr.Zero;
	public IntPtr ob_type = IntPtr.Zero;
	public IntPtr ob_size = IntPtr.Zero;
	public IntPtr tp_name = IntPtr.Zero;
	public IntPtr tp_basicsize = IntPtr.Zero;
	public IntPtr tp_itemsize = IntPtr.Zero;
	public IntPtr tp_dealloc = IntPtr.Zero;
	public IntPtr tp_print = IntPtr.Zero;
	public IntPtr tp_getattr = IntPtr.Zero;
	public IntPtr tp_setattr = IntPtr.Zero;
	public IntPtr tp_compare = IntPtr.Zero;
	public IntPtr tp_repr = IntPtr.Zero;
	public IntPtr tp_as_number = IntPtr.Zero;
	public IntPtr tp_as_sequence = IntPtr.Zero;
	public IntPtr tp_as_mapping = IntPtr.Zero;
	public IntPtr tp_hash = IntPtr.Zero;
	public IntPtr tp_call = IntPtr.Zero;
	public IntPtr tp_str = IntPtr.Zero;
	public IntPtr tp_getattro = IntPtr.Zero;
	public IntPtr tp_setattro = IntPtr.Zero;
	public IntPtr tp_as_buffer = IntPtr.Zero;
	public IntPtr tp_flags = IntPtr.Zero;
	public IntPtr tp_doc = IntPtr.Zero;
	public IntPtr tp_traverse = IntPtr.Zero;
	public IntPtr tp_clear = IntPtr.Zero;
	public IntPtr tp_richcompare = IntPtr.Zero;
	public IntPtr tp_weaklistoffset = IntPtr.Zero;
	public IntPtr tp_iter = IntPtr.Zero;
	public IntPtr tp_iternext = IntPtr.Zero;
	public IntPtr tp_methods = IntPtr.Zero;
	public IntPtr tp_members = IntPtr.Zero;
	public IntPtr tp_getset = IntPtr.Zero;
	public IntPtr tp_base = IntPtr.Zero;
	public IntPtr tp_dict = IntPtr.Zero;
	public IntPtr tp_descr_get = IntPtr.Zero;
	public IntPtr tp_descr_set = IntPtr.Zero;
	public IntPtr tp_dictoffset = IntPtr.Zero;
	public IntPtr tp_init = IntPtr.Zero;
	public IntPtr tp_alloc = IntPtr.Zero;
	public IntPtr tp_new = IntPtr.Zero;
	public IntPtr tp_free = IntPtr.Zero;
	public IntPtr tp_is_gc = IntPtr.Zero;
	public IntPtr tp_bases = IntPtr.Zero;
	public IntPtr tp_mro = IntPtr.Zero;
	public IntPtr tp_cache = IntPtr.Zero;
	public IntPtr tp_subclasses = IntPtr.Zero;
	public IntPtr tp_weaklist = IntPtr.Zero;
	//    }

	//    [StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
	//    internal class PyNumberMethods {
	public IntPtr nb_add = IntPtr.Zero;
	public IntPtr nb_subtract = IntPtr.Zero;
	public IntPtr nb_multiply = IntPtr.Zero;
	public IntPtr nb_divide = IntPtr.Zero;
	public IntPtr nb_remainder = IntPtr.Zero;
	public IntPtr nb_divmod = IntPtr.Zero;
	public IntPtr nb_power = IntPtr.Zero;
	public IntPtr nb_negative = IntPtr.Zero;
	public IntPtr nb_positive = IntPtr.Zero;
	public IntPtr nb_absolute = IntPtr.Zero;
	public IntPtr nb_nonzero = IntPtr.Zero;
        public IntPtr nb_invert = IntPtr.Zero;
	public IntPtr nb_lshift = IntPtr.Zero;
	public IntPtr nb_rshift = IntPtr.Zero;
	public IntPtr nb_and = IntPtr.Zero;
	public IntPtr nb_xor = IntPtr.Zero;
	public IntPtr nb_or = IntPtr.Zero;
        public IntPtr nb_coerce = IntPtr.Zero;
	public IntPtr nb_int = IntPtr.Zero;
	public IntPtr nb_long = IntPtr.Zero;
	public IntPtr nb_float = IntPtr.Zero;
	public IntPtr nb_oct = IntPtr.Zero;
	public IntPtr nb_hex = IntPtr.Zero;
	public IntPtr nb_inplace_add = IntPtr.Zero;
	public IntPtr nb_inplace_subtract = IntPtr.Zero;
	public IntPtr nb_inplace_multiply = IntPtr.Zero;
	public IntPtr nb_inplace_divide = IntPtr.Zero;
	public IntPtr nb_inplace_remainder = IntPtr.Zero;
	public IntPtr nb_inplace_power = IntPtr.Zero;
	public IntPtr nb_inplace_lshift = IntPtr.Zero;
	public IntPtr nb_inplace_rshift = IntPtr.Zero;
	public IntPtr nb_inplace_and = IntPtr.Zero;
	public IntPtr nb_inplace_xor = IntPtr.Zero;
	public IntPtr nb_inplace_or = IntPtr.Zero;
	public IntPtr nb_floor_divide = IntPtr.Zero;
	public IntPtr nb_true_divide = IntPtr.Zero;
	public IntPtr nb_inplace_floor_divide = IntPtr.Zero;
	public IntPtr nb_inplace_true_divide = IntPtr.Zero;
	//    }

	//    [StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
	//    internal class PySequenceMethods {
	public IntPtr sq_length = IntPtr.Zero;
	public IntPtr sq_concat = IntPtr.Zero;
	public IntPtr sq_repeat = IntPtr.Zero;
	public IntPtr sq_item = IntPtr.Zero;
	public IntPtr sq_slice = IntPtr.Zero;
	public IntPtr sq_ass_item = IntPtr.Zero;
	public IntPtr sq_ass_slice = IntPtr.Zero;
	public IntPtr sq_contains = IntPtr.Zero;
	public IntPtr sq_inplace_concat = IntPtr.Zero;
	public IntPtr sq_inplace_repeat = IntPtr.Zero;
	//    }

	//    [StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
	//    internal class PyMappingMethods {
	public IntPtr mp_length = IntPtr.Zero;
	public IntPtr mp_subscript = IntPtr.Zero;
	public IntPtr mp_ass_subscript = IntPtr.Zero;
	//    }

	//    [StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
	//    internal class PyBufferMethods {
	public IntPtr bf_getreadbuffer = IntPtr.Zero;
	public IntPtr bf_getwritebuffer = IntPtr.Zero;
	public IntPtr bf_getsegcount = IntPtr.Zero;
	public IntPtr bf_getcharbuffer = IntPtr.Zero;
    }






    internal enum PyTypeFlags {
	HaveGetCharBuffer = (1 << 0),
	HaveSequenceIn = (1 << 1),
	GC = 0,
	HaveInPlaceOps = (1 << 3),
	CheckTypes = (1 << 4),
	HaveRichCompare = (1 << 5),
	HaveWeakRefs = (1 << 6),
	HaveIter = (1 << 7),
	HaveClass = (1 << 8),
	HeapType = (1 << 9),
	BaseType = (1 << 10),
	Ready = (1 << 12),
	Readying = (1 << 13),
	HaveGC = (1 << 14),
	Managed = (1 << 30),
	Default = HaveGetCharBuffer |
	          HaveSequenceIn |
	          HaveInPlaceOps |
	          HaveRichCompare |
	          HaveWeakRefs | 
                  HaveIter |
                  HaveClass | 0
    }



    // This class defines the function prototypes (delegates) used for low
    // level integration with the CPython runtime. It also provides name 
    // based lookup of the correct prototype for a particular slot name.

    internal class Prototypes {

	static Hashtable pmap;

	static Prototypes() {

	    // Here we build a mapping of PyTypeObject slot names to the
	    // appropriate prototype (delegate) type to use for the slot.

	    Type[] items = typeof(Prototypes).GetNestedTypes();
	    Hashtable p = new Hashtable();

	    for (int i = 0; i < items.Length; i++) {
		Type item = items[i];
		p[item.Name] = item;
	    }

	    pmap = new Hashtable();

	    pmap["tp_dealloc"] = p["DestructorFunc"];
	    pmap["tp_print"] = p["PrintFunc"];
	    pmap["tp_getattr"] = p["BinaryFunc"];
	    pmap["tp_setattr"] = p["ObjObjArgFunc"];
	    pmap["tp_compare"] = p["ObjObjFunc"];
	    pmap["tp_repr"] = p["UnaryFunc"];
	    pmap["tp_hash"] = p["UnaryFunc"];
	    pmap["tp_call"] = p["TernaryFunc"];
	    pmap["tp_str"] = p["UnaryFunc"];
	    pmap["tp_getattro"] = p["BinaryFunc"];
	    pmap["tp_setattro"] = p["ObjObjArgFunc"];
	    pmap["tp_traverse"] = p["ObjObjArgFunc"];
	    pmap["tp_clear"] = p["InquiryFunc"];
	    pmap["tp_richcompare"] = p["RichCmpFunc"];
	    pmap["tp_iter"] = p["UnaryFunc"];
	    pmap["tp_iternext"] = p["UnaryFunc"];
	    pmap["tp_descr_get"] = p["TernaryFunc"];
	    pmap["tp_descr_set"] = p["ObjObjArgFunc"];
	    pmap["tp_init"] = p["ObjObjArgFunc"];
	    pmap["tp_alloc"] = p["IntArgFunc"];
	    pmap["tp_new"] = p["TernaryFunc"];
	    pmap["tp_free"] = p["DestructorFunc"];
	    pmap["tp_is_gc"] = p["InquiryFunc"];

	    pmap["nb_add"] = p["BinaryFunc"];
	    pmap["nb_subtract"] = p["BinaryFunc"];
	    pmap["nb_multiply"] = p["BinaryFunc"];
	    pmap["nb_divide"] = p["BinaryFunc"];
	    pmap["nb_remainder"] = p["BinaryFunc"];
	    pmap["nb_divmod"] = p["BinaryFunc"];
	    pmap["nb_power"] = p["TernaryFunc"];
	    pmap["nb_negative"] = p["UnaryFunc"];
	    pmap["nb_positive"] = p["UnaryFunc"];
	    pmap["nb_absolute"] = p["UnaryFunc"];
	    pmap["nb_nonzero"] = p["InquiryFunc"];
	    pmap["nb_invert"] = p["UnaryFunc"];
	    pmap["nb_lshift"] = p["BinaryFunc"];
	    pmap["nb_rshift"] = p["BinaryFunc"];
	    pmap["nb_and"] = p["BinaryFunc"];
	    pmap["nb_xor"] = p["BinaryFunc"];
	    pmap["nb_or"] = p["BinaryFunc"];
	    pmap["nb_coerce"] = p["ObjObjFunc"];
	    pmap["nb_int"] = p["UnaryFunc"];
	    pmap["nb_long"] = p["UnaryFunc"];
	    pmap["nb_float"] = p["UnaryFunc"];
	    pmap["nb_oct"] = p["UnaryFunc"];
	    pmap["nb_hex"] = p["UnaryFunc"];
	    pmap["nb_inplace_add"] = p["BinaryFunc"];
	    pmap["nb_inplace_subtract"] = p["BinaryFunc"];
	    pmap["nb_inplace_multiply"] = p["BinaryFunc"];
	    pmap["nb_inplace_divide"] = p["BinaryFunc"];
	    pmap["nb_inplace_remainder"] = p["BinaryFunc"];
	    pmap["nb_inplace_power"] = p["TernaryFunc"];
	    pmap["nb_inplace_lshift"] = p["BinaryFunc"];
	    pmap["nb_inplace_rshift"] = p["BinaryFunc"];
	    pmap["nb_inplace_and"] = p["BinaryFunc"];
	    pmap["nb_inplace_xor"] = p["BinaryFunc"];
	    pmap["nb_inplace_or"] = p["BinaryFunc"];
	    pmap["nb_floor_divide"] = p["BinaryFunc"];
	    pmap["nb_true_divide"] = p["BinaryFunc"];
	    pmap["nb_inplace_floor_divide"] = p["BinaryFunc"];
	    pmap["nb_inplace_true_divide"] = p["BinaryFunc"];

	    pmap["sq_length"] = p["InquiryFunc"];
	    pmap["sq_concat"] = p["BinaryFunc"];
	    pmap["sq_repeat"] = p["IntArgFunc"];
	    pmap["sq_item"] = p["IntArgFunc"];
	    pmap["sq_slice"] = p["IntIntArgFunc"];
	    pmap["sq_ass_item"] = p["IntObjArgFunc"];
	    pmap["sq_ass_slice"] = p["IntIntObjArgFunc"];
	    pmap["sq_contains"] = p["ObjObjFunc"];
	    pmap["sq_inplace_concat"] = p["BinaryFunc"];
	    pmap["sq_inplace_repeat"] = p["IntArgFunc"];

	    pmap["mp_length"] = p["InquiryFunc"];
	    pmap["mp_subscript"] = p["BinaryFunc"];
	    pmap["mp_ass_subscript"] = p["ObjObjArgFunc"];
	    
	    pmap["bf_getreadbuffer"] = p["IntObjArgFunc"];
	    pmap["bf_getwritebuffer"] = p["IntObjArgFunc"];
	    pmap["bf_getsegcount"] = p["ObjObjFunc"];
	    pmap["bf_getcharbuffer"] = p["IntObjArgFunc"];
	}

	internal static Type GetPrototype(string name) {
	    return pmap[name] as Type;
	}

	[CallConvCdecl()]
	public delegate IntPtr UnaryFunc(IntPtr ob);

	[CallConvCdecl()]
	public delegate IntPtr BinaryFunc(IntPtr ob, IntPtr arg);

	[CallConvCdecl()]
	public delegate IntPtr TernaryFunc(IntPtr ob, IntPtr a1, IntPtr a2);

	[CallConvCdecl()]
	public delegate int InquiryFunc(IntPtr ob);

	[CallConvCdecl()]
	public delegate IntPtr IntArgFunc(IntPtr ob, int arg);

	[CallConvCdecl()]
	public delegate IntPtr IntIntArgFunc(IntPtr ob, int a1, int a2);

	[CallConvCdecl()]
	public delegate int IntObjArgFunc(IntPtr ob, int a1, IntPtr a2);

	[CallConvCdecl()]
	public delegate int IntIntObjArgFunc(IntPtr o, int a, int b, IntPtr c);

	[CallConvCdecl()]
	public delegate int ObjObjArgFunc(IntPtr o, IntPtr a, IntPtr b);

	[CallConvCdecl()]
	public delegate int ObjObjFunc(IntPtr ob, IntPtr arg);

	[CallConvCdecl()]
	public delegate void DestructorFunc(IntPtr ob);

	[CallConvCdecl()]
	public delegate int PrintFunc(IntPtr ob, IntPtr a, int b);

	[CallConvCdecl()]
	public delegate IntPtr RichCmpFunc(IntPtr ob, IntPtr a, int b);

    }


    internal class CallbackThunks {

	[CallConvCdecl()]
	public delegate void tp_dealloc(IntPtr ob);

	[CallConvCdecl()]
	public delegate IntPtr tp_print(IntPtr ob, IntPtr file, IntPtr flags);

	[CallConvCdecl()]
	public delegate IntPtr tp_getattr(IntPtr ob, IntPtr key);

	[CallConvCdecl()]
	public delegate int tp_setattr(IntPtr ob, IntPtr key, IntPtr val);

	[CallConvCdecl()]
	public delegate IntPtr tp_getattro(IntPtr ob, IntPtr key);

	[CallConvCdecl()]
	public delegate int tp_setattro(IntPtr ob, IntPtr key, IntPtr val);

	[CallConvCdecl()]
	public delegate int tp_compare(IntPtr ob, IntPtr other);

	[CallConvCdecl()]
	public delegate IntPtr tp_repr(IntPtr ob);

	[CallConvCdecl()]
	public delegate IntPtr tp_hash(IntPtr ob);

	[CallConvCdecl()]
	public delegate IntPtr tp_call(IntPtr ob, IntPtr args, IntPtr kw);

	[CallConvCdecl()]
	public delegate IntPtr tp_str(IntPtr ob);

	[CallConvCdecl()]
	public delegate IntPtr tp_iter(IntPtr ob);

	[CallConvCdecl()]
	public delegate IntPtr tp_iternext(IntPtr ob);

	[CallConvCdecl()]
	public delegate IntPtr tp_descr_get(IntPtr desc, IntPtr ob, IntPtr tp);

	[CallConvCdecl()]
	public delegate int tp_descr_set(IntPtr desc, IntPtr ob, IntPtr tp);

	[CallConvCdecl()]
	public delegate int tp_init(IntPtr ob, IntPtr args, IntPtr kw);

	[CallConvCdecl()]
	public delegate IntPtr tp_new(IntPtr tp, IntPtr args, IntPtr kw);

	[CallConvCdecl()]
	public delegate IntPtr tp_alloc(IntPtr tp, int sz);

	[CallConvCdecl()]
	public delegate void tp_free(IntPtr ob);

	[CallConvCdecl()]
	public delegate int tp_is_gc(IntPtr type);


	// PyNumberMethods prototypes

	[CallConvCdecl()]
	public delegate IntPtr nb_add(IntPtr ob, IntPtr arg);

	[CallConvCdecl()]
	public delegate IntPtr nb_subtract(IntPtr ob, IntPtr arg);

	[CallConvCdecl()]
	public delegate IntPtr nb_multiply(IntPtr ob, IntPtr arg);

	[CallConvCdecl()]
	public delegate IntPtr nb_divide(IntPtr ob, IntPtr arg);

	[CallConvCdecl()]
	public delegate IntPtr nb_remainder(IntPtr ob, IntPtr arg);

	[CallConvCdecl()]
	public delegate IntPtr nb_divmod(IntPtr ob, IntPtr arg);

	[CallConvCdecl()]
	public delegate IntPtr nb_power(IntPtr ob, IntPtr arg, IntPtr arg2);

	[CallConvCdecl()]
	public delegate IntPtr nb_negative(IntPtr ob);

	[CallConvCdecl()]
	public delegate IntPtr nb_positive(IntPtr ob);

	[CallConvCdecl()]
	public delegate IntPtr nb_absolute(IntPtr ob);



	[CallConvCdecl()]
	public delegate IntPtr nb_inplace_add(IntPtr ob, IntPtr arg);

	[CallConvCdecl()]
	public delegate IntPtr nb_inplace_subtract(IntPtr ob, IntPtr arg);


	[CallConvCdecl()]
	public delegate IntPtr generic(IntPtr ob, IntPtr args, IntPtr kw);
	
    }


    [StructLayout(LayoutKind.Sequential, CharSet=CharSet.Ansi)]
    internal struct CallbackThunk {
	public Delegate fn;

	public CallbackThunk(Delegate d) {
	    fn = d;
	}
    }


    // Bleah. We can't call function pointers directly from the CLR.
    // This lets us call certain type slots by wrapping the slot in
    // a PyCFunction and invoking it through the Python runtime.

    internal class SlotMethod {

	public static IntPtr mdef;

	static SlotMethod() {
	    mdef = Marshal.AllocHGlobal(4 * IntPtr.Size);
	}

	public static IntPtr Call(IntPtr fn, IntPtr self, IntPtr args) { 
	    Marshal.WriteIntPtr(mdef, (1 * IntPtr.Size), fn);
	    Marshal.WriteIntPtr(mdef, (2 * IntPtr.Size), (IntPtr)0x0001);
	    IntPtr func = Runtime.PyCFunction_New(mdef, self);
	    IntPtr result = Runtime.PyObject_Call(func, args, IntPtr.Zero);
	    Runtime.Decref(func);
	    return result;
	}

	public static IntPtr Call(IntPtr fn, IntPtr self, IntPtr args, 
				  IntPtr kw) { 
	    Marshal.WriteIntPtr(mdef, (1 * IntPtr.Size), fn);
	    Marshal.WriteIntPtr(mdef, (2 * IntPtr.Size), (IntPtr)0x0002);
	    IntPtr func = Runtime.PyCFunction_New(mdef, self);
	    IntPtr result = Runtime.PyObject_Call(func, args, kw);
	    Runtime.Decref(func);
	    return result;
	}

	public static IntPtr Call(IntPtr fn, IntPtr self) { 
	    Marshal.WriteIntPtr(mdef, (1 * IntPtr.Size), fn);
	    Marshal.WriteIntPtr(mdef, (2 * IntPtr.Size), (IntPtr)0x0004);
	    IntPtr args = Runtime.PyTuple_New(0);
	    IntPtr func = Runtime.PyCFunction_New(mdef, self);
	    IntPtr result = Runtime.PyObject_Call(func, args, IntPtr.Zero);
	    Runtime.Decref(func);
	    Runtime.Decref(args);
	    return result;
	}

    }


    // A NativeMethod object wraps a (static) managed method, making it
    // look like a PyCFunction object. This is used mainly to implement
    // the CLR import hook.

    internal class NativeMethod {

	public Delegate mDelegate;
	public IntPtr methodDef;
	public IntPtr pyHandle;

	public NativeMethod(Type obType, string name) {

	    // Allocate and initialize a PyMethodDef structure to represent 
	    // the managed method, then create a PyCFunction. The PyMethodDef 
	    // will be cleaned up when the NativeMethod is garbage-collected.

	    mDelegate = Delegate.CreateDelegate(typeof(CallbackThunks.generic),
						obType.GetMethod(name)
						);
	    CallbackThunk cb = new CallbackThunk(mDelegate);
	    IntPtr pThunk = Marshal.AllocHGlobal(IntPtr.Size);
	    Marshal.StructureToPtr(cb, pThunk, false);
	    IntPtr fp = Marshal.ReadIntPtr(pThunk);
	    Marshal.FreeHGlobal(pThunk);

	    methodDef = Runtime.PyMem_Malloc(4 * IntPtr.Size);
	    Marshal.WriteIntPtr(methodDef, Marshal.StringToHGlobalAnsi(name));
	    Marshal.WriteIntPtr(methodDef, (1 * IntPtr.Size), fp);
	    Marshal.WriteIntPtr(methodDef, (2 * IntPtr.Size), (IntPtr)0x0002);
	    Marshal.WriteIntPtr(methodDef, (3 * IntPtr.Size), IntPtr.Zero);
	    pyHandle = Runtime.PyCFunction_New(methodDef, IntPtr.Zero);

	}

	public IntPtr Call(IntPtr args, IntPtr kw) {
	    return Runtime.PyCFunction_Call(pyHandle, args, kw);
	}

    }









}
