// Copyright (c) 2001, 2002 Zope Corporation and Contributors.
//
// All Rights Reserved.
//
// This software is subject to the provisions of the Zope Public License,
// Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
// WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
// FOR A PARTICULAR PURPOSE.

using System;
using System.Threading;
using System.Runtime.InteropServices;
using System.Runtime.CompilerServices;
using System.Collections;
using System.Reflection;
using System.Reflection.Emit;

namespace Python.Runtime {

    // todo: is this actually needed??

    /// <summary>
    /// The CastHelper class does low-level indirect casting of objects.
    /// </summary>

    internal class CastHelper {

	static Hashtable cache;

	static CastHelper() {
	    cache = new Hashtable();
	}

	internal static Object CastObject(Object ob, Type type) {
	    ICastHelper helper = cache[type.FullName] as ICastHelper;
	    if (helper == null) {
		helper = GetCastHelper(type);
		cache[type.FullName] = helper;
	    }
	    return helper.CastObject(ob);
	}

	// todo: avoid using instances

	private static ICastHelper GetCastHelper(Type type) {

	    string name = "__CastHelper_" + type.FullName;
	    name = name.Replace(Type.Delimiter, '_');
	    TypeBuilder tb = CodeGenerator.DefineType(name);

	    Type basetype = typeof(ICastHelper);
	    tb.AddInterfaceImplementation(basetype);

	    MethodInfo mi = basetype.GetMethod("CastObject");
	    
	    ParameterInfo[] pi = mi.GetParameters();
	    int count = pi.Length;

	    Type[] args = new Type[count];
	    for (int i = 0; i < count; i++) {
		args[i] = pi[i].ParameterType;
	    }

	    MethodBuilder mb = tb.DefineMethod(mi.Name,
					       MethodAttributes.Public |
					       MethodAttributes.Virtual,
					       mi.ReturnType,
					       args
					       );

	    ILGenerator il = mb.GetILGenerator();
	    il.Emit(OpCodes.Ldarg_1);
	    il.Emit(OpCodes.Castclass, type);
	    il.Emit(OpCodes.Ret);

	    tb.DefineMethodOverride(mb, mi);

	    Type impl = tb.CreateType();
	    ICastHelper helper = (ICastHelper)Activator.CreateInstance(impl);
	    return helper;
	}

    }

    public interface ICastHelper {

	object CastObject(object ob);

    }



}
