// Copyright (c) 2001, 2002 Zope Corporation and Contributors.
//
// All Rights Reserved.
//
// This software is subject to the provisions of the Zope Public License,
// Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
// WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
// FOR A PARTICULAR PURPOSE.

using System;
using System.Runtime.InteropServices;
using System.Collections;
using System.Reflection;

namespace Python.Runtime {

    /// <summary>
    /// The ClassManager is responsible for creating and managing Python
    /// types that reflect managed classes, structs, interfaces, enums
    /// and delegates.
    /// </summary>

    internal class ClassManager {

	static Hashtable cache;
	static Type dtype;

	private ClassManager() {}

	static ClassManager() {
	    cache = new Hashtable();
	    dtype = typeof(System.Delegate);
	}

	/// <summary>
	/// GetClass Method
	/// </summary>
	///
	/// <remarks>
	/// Return a ClassObject that reflects the given CLR type, or null
	/// if the ClassObject cannot be created. Ref behavior??
	/// </remarks>

	internal static ClassBase GetClass(Type type) {
	    Object ob = cache[type];
	    if (ob == null) {
		ClassBase c = CreateClass(type);
		cache.Add(type, c);
		return c;
	    }
	    return (ClassBase) ob;
	}

	private static ClassBase CreateClass(Type type) {
	    if (type.IsSubclassOf(dtype)) {
		return new DelegateObject(type);
	    }

	    if (type.IsArray) {
		return new ArrayObject(type);
	    }

	    if (type.IsInterface) {
		return new InterfaceObject(type);
	    }

	    return new ClassObject(type);

	}

    }	

}
