// Copyright (c) 2001, 2002 Zope Corporation and Contributors.
//
// All Rights Reserved.
//
// This software is subject to the provisions of the Zope Public License,
// Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
// WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
// FOR A PARTICULAR PURPOSE.

using System;

namespace Python.Runtime {

    //========================================================================
    // Implements a Python event binding type, similar to a method binding.
    //========================================================================

    internal class EventBinding : ExtensionType {

	EventObject e;
	IntPtr target;

	public EventBinding(EventObject e, IntPtr target) : base() {
	    Runtime.Incref(target);
	    this.target = target;
	    this.e = e;
	}

	//====================================================================
	// EventBinding += operator implementation.
	//====================================================================

	[CallConvCdecl()]
	public static IntPtr nb_inplace_add(IntPtr ob, IntPtr arg) {
	    EventBinding self = (EventBinding)GetManagedObject(ob);
	    int result = self.e.AddEventHandler(self.target, arg);
	    if (result == -1) {
		return IntPtr.Zero;
	    }
	    Runtime.Incref(self.Handle);
	    return self.Handle;
	}

	//====================================================================
	// EventBinding -= operator implementation.
	//====================================================================

	[CallConvCdecl()]
	public static IntPtr nb_inplace_subtract(IntPtr ob, IntPtr arg) {
	    EventBinding self = (EventBinding)GetManagedObject(ob);
	    int result = self.e.RemoveEventHandler(self.target, arg);
	    if (result == -1) {
		return IntPtr.Zero;
	    }
	    Runtime.Incref(self.Handle);
	    return self.Handle;
	}

	//====================================================================
	// EventBinding __call__ implementation.
	//====================================================================

	[CallConvCdecl()]
	public static IntPtr tp_call(IntPtr ob, IntPtr args, IntPtr kw) {
	    EventBinding self = (EventBinding)GetManagedObject(ob);
	    return self.e.Invoke(self.target, args, kw);
	}

	//====================================================================
	// EventBinding __repr__ implementation.
	//====================================================================

	[CallConvCdecl()]
	public static IntPtr tp_repr(IntPtr ob) {
	    EventBinding self = (EventBinding)GetManagedObject(ob);
	    string type = (self.target == IntPtr.Zero) ? "unbound" : "bound";
	    string s = String.Format("<{0} event '{1}'>", type, self.e.name);
	    return Runtime.PyString_FromStringAndSize(s, s.Length);
	}

	//====================================================================
	// EventBinding dealloc implementation.
	//====================================================================

	[CallConvCdecl()]
	public static new void tp_dealloc(IntPtr ob) {
	    EventBinding self = (EventBinding)GetManagedObject(ob);
	    Runtime.Decref(self.target);
	    ExtensionType.FinalizeObject(self);
	}

    }


}
