// Copyright (c) 2001, 2002 Zope Corporation and Contributors.
//
// All Rights Reserved.
//
// This software is subject to the provisions of the Zope Public License,
// Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
// WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
// FOR A PARTICULAR PURPOSE.

using System;
using System.Reflection;

namespace Python.Runtime {

    /// <summary>
    /// Encapsulates the Python exception APIs.
    /// </summary>

    public class Exceptions {

	private Exceptions() {}

	internal static void Initialize() {
	    // TODO: fix this to use PyObject wrappers correctly (refs!).

	    // This is called when the Python runtime is intitialized. It 
	    // uses reflection to generate PyObject instances to wrap the
	    // standard Python exception types.
	    IntPtr module = Runtime.PyImport_ImportModule("exceptions");

	    Type tp = typeof(Exceptions);
	    foreach (FieldInfo fi in tp.GetFields(BindingFlags.Public | 
						  BindingFlags.Static)) {
		IntPtr op = Runtime.PyObject_GetAttrString(module, fi.Name);
		if (op != IntPtr.Zero) {
		    // owned reference
		    PyObject ob = new PyObject(op);
		    fi.SetValue(tp, ob);
		}
	    }
	    Runtime.Decref(module);
	    Runtime.PyErr_Clear();
	}

	/// <summary>
	/// GetException Method
	/// </summary>
	///
	/// <remarks>
	/// Retrieve Python exception information as a PythonException
	/// instance. The properties of the PythonException may be used
	/// to access the exception type, value and traceback info.
	/// </remarks>

	public static PythonException GetException() {
	    // TODO: implement this.
	    return null;
	}

	/// <summary>
	/// ExceptionMatches Method
	/// </summary>
	///
	/// <remarks>
	/// Returns true if the current Python exception matches the given
	/// Python object. This is a wrapper for PyErr_ExceptionMatches.
	/// </remarks>

	public static bool ExceptionMatches(PyObject ob) {
	    return Runtime.PyErr_ExceptionMatches(ob.Handle) != 0;
	}

	/// <summary>
	/// ExceptionMatches Method
	/// </summary>
	///
	/// <remarks>
	/// Returns true if the given Python exception matches the given
	/// Python object. This is a wrapper for PyErr_GivenExceptionMatches.
	/// </remarks>

	public static bool ExceptionMatches(PyObject exc, PyObject ob) {
	    int i = Runtime.PyErr_GivenExceptionMatches(exc.Handle, ob.Handle);
	    return (i != 0);
	}

	/// <summary>
	/// SetError Method
	/// </summary>
	///
	/// <remarks>
	/// Sets the current Python exception given a native string.
	/// This is a wrapper for the Python PyErr_SetString call.
	/// </remarks>

	public static void SetError(PyObject ob, string value) {
	    Runtime.PyErr_SetString(ob.Handle, value);
	}

	/// <summary>
	/// SetError Method
	/// </summary>
	///
	/// <remarks>
	/// Sets the current Python exception given a Python object.
	/// This is a wrapper for the Python PyErr_SetObject call.
	/// </remarks>

	public static void SetError(PyObject ob, PyObject value) {
	    Runtime.PyErr_SetObject(ob.Handle, value.Handle);
	}

	/// <summary>
	/// SetError Method
	/// </summary>
	///
	/// <remarks>
	/// Sets the current Python exception given a CLR exception
	/// object. The CLR exception instance is wrapped as a Python
	/// object, allowing it to be handled naturally from Python.
	/// </remarks>

	public static void SetError(Exception e) {
	  Runtime.PyErr_Clear();
	    IntPtr op = CLRObject.GetInstHandle(e);
	    Runtime.Incref(op);
	    Runtime.Incref(op);
	    Runtime.PyErr_SetString(op, e.Message);
	}

	/// <summary>
	/// SetFromErrno Method
	/// </summary>
	///
	/// <remarks>
	/// Sets the current Python exception based on the C errno.
	/// This is a wrapper for the Python PyErr_SetFromErrno call.
	/// </remarks>

	public static void SetFromErrno(PyObject ob) {
	    Runtime.PyErr_SetFromErrno(ob.Handle);
	}

	/// <summary>
	/// ErrorOccurred Method
	/// </summary>
	///
	/// <remarks>
	/// Returns true if an exception occurred in the Python runtime.
	/// This is a wrapper for the Python PyErr_Occurred call.
	/// </remarks>

	public static bool ErrorOccurred() {
	    return Runtime.PyErr_Occurred() != 0;
	}

	/// <summary>
	/// Clear Method
	/// </summary>
	///
	/// <remarks>
	/// Clear any exception that has been set in the Python runtime.
	/// </remarks>

	public static void Clear() {
	    Runtime.PyErr_Clear();
	}

	/// <summary>
	/// Restore Method
	/// </summary>
	///
	/// <remarks>
	/// Set Python exception information from the given Python objects.
	/// This is a wrapper for the Python PyErr_Restore call.
	/// </remarks>

	public static void Restore(PyObject type, PyObject val, PyObject tb) {
	    Runtime.PyErr_Restore(type.Handle, val.Handle, tb.Handle);
	}


	public static PyObject ArithmeticError;
	public static PyObject AssertionError;
	public static PyObject AttributeError;
	public static PyObject DeprecationWarning;
	public static PyObject EOFError;
	public static PyObject EnvironmentError;
	public static PyObject Exception;
	public static PyObject FloatingPointError;
	public static PyObject IOError;
	public static PyObject ImportError;
	public static PyObject IndentationError;
	public static PyObject IndexError;
	public static PyObject KeyError;
	public static PyObject KeyboardInterrupt;
	public static PyObject LookupError;
	public static PyObject MemoryError;
	public static PyObject NameError;
	public static PyObject NotImplementedError;
	public static PyObject OSError;
	public static PyObject OverflowError;
	public static PyObject OverflowWarning;
	public static PyObject ReferenceError;
	public static PyObject RuntimeError;
	public static PyObject RuntimeWarning;
	public static PyObject StandardError;
	public static PyObject StopIteration;
	public static PyObject SyntaxError;
	public static PyObject SyntaxWarning;
	public static PyObject SystemError;
	public static PyObject SystemExit;
	public static PyObject TabError;
	public static PyObject TypeError;
	public static PyObject UnboundLocalError;
	public static PyObject UnicodeError;
	public static PyObject UserWarning;
	public static PyObject ValueError;
	public static PyObject Warning;
	public static PyObject WindowsError;
	public static PyObject ZeroDivisionError;

    }


}
