// Copyright (c) 2001, 2002 Zope Corporation and Contributors.
//
// All Rights Reserved.
//
// This software is subject to the provisions of the Zope Public License,
// Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
// WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
// FOR A PARTICULAR PURPOSE.

using System;
using System.Runtime.InteropServices;
using System.Collections;
using System.Reflection;

namespace Python.Runtime {

    //========================================================================
    // Base class for extensions whose instances *share* a single Python
    // type object, such as the types that represent CLR methods, fields, 
    // etc. Instances implemented by this class do not support subtyping.
    //========================================================================

    internal abstract class ExtensionType : ManagedType {

	public ExtensionType() : base() {

	    // Get the Python type pointer for this extension type. This
	    // will return a cached handle after the first call per type.

	    this.tpHandle = TypeManager.GetTypeHandle(this.GetType());

	    // Create a PyObject struct to associate this object with Python.
	    // We store a GCHandle to the CLR object in the PyObject struct  
	    // and store the PyObject * in the CLR object, which lets us find 
	    // the associated object given either the Python or CLR version.

	    PyObjectHead pyObj = new PyObjectHead();
	    this.gcHandle = GCHandle.Alloc(this);
	    pyObj.ob_data = (IntPtr)this.gcHandle;
	    pyObj.ob_refcnt = (IntPtr) 1;
	    pyObj.ob_type = tpHandle;
	    Runtime.Incref(tpHandle);

	    // Now we blit this to unmanaged memory to avoid having to pin
	    // CLR objects, which would interfere with the managed GC.

	    this.pyHandle = Runtime.PyMem_Malloc((3 * IntPtr.Size));
	    GCHandle gch = GCHandle.Alloc(pyObj, GCHandleType.Pinned);
	    Marshal.StructureToPtr(pyObj, pyHandle, false);
	    gch.Free();
	}


	[CallConvCdecl()]
	public static int tp_is_gc(IntPtr type) {
	    return 0;
	}

	[CallConvCdecl()]
	public static int tp_setattro(IntPtr ob, IntPtr key, IntPtr val) {
	    string message = "type does not support setting attributes";
	    if (val == IntPtr.Zero) {
		message = "readonly attribute";
	    }
	    Exceptions.SetError(Exceptions.TypeError, message);
	    return -1;
	}

	[CallConvCdecl()]
	public static void tp_dealloc(IntPtr ob) {
	    // Clean up a Python instance of this extension type. This 
	    // frees the allocated Python object and decrefs the type.
	    ManagedType self = GetManagedObject(ob);
	    FinalizeObject(self);
	}

	public static void FinalizeObject(ManagedType self) {
	    // Perform standard base-class level finalization.
	    Runtime.PyMem_Free(self.pyHandle);
	    Runtime.Decref(self.tpHandle);
	    self.gcHandle.Free();
	}



    }


}
