// Copyright (c) 2003 Zope Corporation and Contributors.
//
// All Rights Reserved.
//
// This software is subject to the provisions of the Zope Public License,
// Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
// WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
// FOR A PARTICULAR PURPOSE.

using System;
using System.Reflection;

namespace Python.Runtime {

    /// <summary>
    /// Provides the implementation for reflected interface types. Managed
    /// interfaces are represented in Python by actual Python type objects.
    /// Each of those type objects is associated with an instance of this
    /// class, which provides the implementation for the Python type.
    /// </summary>

    internal class InterfaceObject : ClassBase {

	internal InterfaceObject(Type tp) : base(tp) {}

	//====================================================================
	// Implements __new__ for reflected interface types.
	//====================================================================

	[CallConvCdecl()]
	public static IntPtr tp_new(IntPtr tp, IntPtr args, IntPtr kw) {
	    InterfaceObject self = (InterfaceObject)GetManagedObject(tp);

	    if (Runtime.PyTuple_Size(args) != 1) {
		Exceptions.SetError(Exceptions.TypeError,
				    "interface takes exactly one argument"
				    );
		return IntPtr.Zero;
	    }

	    IntPtr inst = Runtime.PyTuple_GetItem(args, 0);
	    CLRObject co = GetManagedObject(inst) as CLRObject;
	    Type type = self.type;

	    if ((co == null) || (!type.IsInstanceOfType(co.inst))) {
		string msg = "object does not implement " + type.Name;
		Exceptions.SetError(Exceptions.TypeError, msg);
		return IntPtr.Zero;
	    }

	    Object obj = co.inst;

	    if (obj.GetType() != type) {
		// this probably isn't needed anymore...
		obj = CastHelper.CastObject(obj, type);
	    }

	    IntPtr op = CLRObject.GetInstHandle(obj, self.pyHandle);
	    Runtime.Incref(op);
	    return op;
	}

	//====================================================================
	// Implements __init__ for reflected interface types.
	//====================================================================

	[CallConvCdecl()]
	public static int tp_init(IntPtr ob, IntPtr args, IntPtr kw) {
	    return 0;
	}

	/*
	public override IntPtr ClassRepr() {
	    string s = String.Format("<delegate '{0}'>", this.type.FullName);
	    return Runtime.PyString_FromStringAndSize(s, s.Length);
	}
	*/




    }	




}
