// Copyright (c) 2001, 2002 Zope Corporation and Contributors.
//
// All Rights Reserved.
//
// This software is subject to the provisions of the Zope Public License,
// Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
// WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
// FOR A PARTICULAR PURPOSE.

using System;
using System.Runtime.InteropServices;
using System.Collections;
using System.Reflection;

namespace Python.Runtime {

    //========================================================================
    // Common base class for all objects that are implemented in managed 
    // code. It defines the common fields that associate CLR and Python
    // objects and common utilities to convert between those identities.
    //========================================================================

    internal abstract class ManagedType {

	internal GCHandle gcHandle; // Native handle
	internal IntPtr pyHandle;   // PyObject *
	internal IntPtr tpHandle;   // PyType *

	public IntPtr Handle {
	    get { return pyHandle; }
	}

	public IntPtr TypeHandle {
	    get { return tpHandle; }
	}

	internal static ManagedType GetManagedObject(IntPtr ob) {

	    // Given a PyObject *, return the managed object associated with
	    // the Python object or null if the object is not managed. We do
	    // this by looking at the __mro__ of the object's type to see if
	    // the object is an instance of a reflected managed type or is a
	    // (direct, or subclass of a) reflected managed type.

	    if (ob == IntPtr.Zero) {
		return null;
	    }

	    IntPtr tp = Marshal.ReadIntPtr(ob, IntPtr.Size);
	    if (tp == Runtime.PyTypeType) {
		tp = ob;
	    }

	    // If ob->ob_type->tp_mro is null then this is not a new style
	    // class (type object), so it cannot be a CLR based object.

	    IntPtr mro = Marshal.ReadIntPtr(tp, (43 * IntPtr.Size));
	    if (mro == IntPtr.Zero) {
		return null;
	    }

	    // This gets called a lot, so this effectively inlines the std
	    // PyTuple_GET_SIZE / GET_ITEM macros to avoid making a lot of
	    // function calls across the managed / unmanaged code boundary.

	    int num = (int)Marshal.ReadIntPtr(mro, (2 * IntPtr.Size));
	    
	    for (int i = 0; i < num; i++) {
		IntPtr item = Marshal.ReadIntPtr(mro, ((3 + i) * IntPtr.Size));
		int flags = (int)Marshal.ReadIntPtr(item, (21 * IntPtr.Size));
		if ((flags & (1 << 30)) != 0) {
		    IntPtr handle = (tp == ob) ?
			Marshal.ReadIntPtr(item, (2 * IntPtr.Size)) :
			Marshal.ReadIntPtr(ob, (2 * IntPtr.Size));
		    GCHandle gch = (GCHandle)handle;
		    return (ManagedType)gch.Target;
		}
	    }

	    return null;
	}


	internal static ManagedType GetManagedObjectErr(IntPtr ob) {
	    ManagedType result = GetManagedObject(ob);
	    if (result == null) {
		Exceptions.SetError(Exceptions.TypeError, 
			   "invalid argument, expected CLR type"
			   );
	    }
	    return result;
	}


	internal static bool IsManagedType(IntPtr ob) {
	    Object result = GetManagedObject(ob);
	    return (result != null);
	}




	public static explicit operator PyObject (ManagedType ob) {
	    return new PyObject(ob.pyHandle);
	}

    }

}

