// Copyright (c) 2001, 2002 Zope Corporation and Contributors.
//
// All Rights Reserved.
//
// This software is subject to the provisions of the Zope Public License,
// Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
// WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
// FOR A PARTICULAR PURPOSE.

using System;
using System.Collections;
using System.Reflection;

namespace Python.Runtime {

    //========================================================================
    // Implements a Python type that provides access to CLR object methods.
    //========================================================================
    // todo: decref unbound on dealloc

    internal class MethodObject : ExtensionType {

	internal MethodInfo[] info;
	internal string name;
	MethodBinding unbound;
	MethodBinder binder;

	public MethodObject(string name, MethodInfo[] info) : base() {
	    this.name = name;
	    this.info = info;
	    binder = new MethodBinder();
	    for (int i = 0; i < info.Length; i++) {
		binder.AddMethod((MethodInfo)info[i]);
	    }
	}

	public virtual IntPtr Invoke(IntPtr target, IntPtr args, IntPtr kw) {
	    // TODO: should probably release GIL around the method call
	    return binder.Invoke(target, args, kw);
	}

	//====================================================================
	// Descriptor __get__ implementation. Accessing a CLR method returns
	// a "bound" method similar to a Python bound method. 
	//====================================================================

	[CallConvCdecl()]
	public static IntPtr tp_descr_get(IntPtr ds, IntPtr ob, IntPtr tp) {
	    MethodObject self = (MethodObject)GetManagedObject(ds);
	    MethodBinding binding;

	    // If the method is accessed through its type (rather than via
	    // an instance) we return an 'unbound' MethodBinding that will
	    // cached for future accesses through the type.

	    if (ob == IntPtr.Zero) {
		if (self.unbound == null) {
		    self.unbound = new MethodBinding(self, IntPtr.Zero);
		}
		binding = self.unbound;
		Runtime.Incref(binding.Handle);;
		return binding.Handle;
	    }

	    if (Runtime.PyObject_IsInstance(ob, tp) != 1) {
		Exceptions.SetError(Exceptions.TypeError, 
				    "invalid descriptor argument"
				    );
		return IntPtr.Zero;
	    }

	    binding = new MethodBinding(self, ob);
	    return binding.Handle;
	}

	//====================================================================
	// Descriptor __repr__ implementation.
	//====================================================================

	[CallConvCdecl()]
	public static IntPtr tp_repr(IntPtr ob) {
	    MethodObject self = (MethodObject)GetManagedObject(ob);
	    string s = String.Format("<method '{0}'>", self.name);
	    return Runtime.PyString_FromStringAndSize(s, s.Length);
	}

	//====================================================================
	// Descriptor dealloc implementation.
	//====================================================================

	[CallConvCdecl()]
	public static new void tp_dealloc(IntPtr ob) {
	    MethodObject self = (MethodObject)GetManagedObject(ob);
	    if (self.unbound != null) {
		Runtime.Decref(self.unbound.Handle);
	    }
	    ExtensionType.FinalizeObject(self);
	}


    }


}
