// Copyright (c) 2001, 2002 Zope Corporation and Contributors.
//
// All Rights Reserved.
//
// This software is subject to the provisions of the Zope Public License,
// Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
// WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
// FOR A PARTICULAR PURPOSE.

using System;
using System.Collections;
using System.Runtime.InteropServices;

namespace Python.Runtime {

    /// <summary>
    /// A StaticMethodWrapper wraps a static method of a managed type,
    /// making it callable by Python as a PyCFunction object. This is
    /// currently used mainly to implement special cases like the CLR
    /// import hook.
    /// </summary>

    internal class StaticMethodWrapper {

	public Delegate mDelegate;
	public IntPtr methodDef;
	public IntPtr pyHandle;

	private static ArrayList keepAlive;

	static StaticMethodWrapper() {
	    keepAlive = new ArrayList();
	}

	public StaticMethodWrapper(Type obType, string name) {

	    // Allocate and initialize a PyMethodDef structure to represent 
	    // the managed method, then create a PyCFunction. 

	    // Currently, these are never released. need to think harder.

	    mDelegate = Delegate.CreateDelegate(typeof(CallbackThunks.generic),
						obType.GetMethod(name)
						);
	    CallbackThunk cb = new CallbackThunk(mDelegate);
	    IntPtr pThunk = Marshal.AllocHGlobal(IntPtr.Size);
	    Marshal.StructureToPtr(cb, pThunk, false);
	    IntPtr fp = Marshal.ReadIntPtr(pThunk);
	    Marshal.FreeHGlobal(pThunk);

	    methodDef = Runtime.PyMem_Malloc(4 * IntPtr.Size);
	    Marshal.WriteIntPtr(methodDef, Marshal.StringToHGlobalAnsi(name));
	    Marshal.WriteIntPtr(methodDef, (1 * IntPtr.Size), fp);
	    Marshal.WriteIntPtr(methodDef, (2 * IntPtr.Size), (IntPtr)0x0002);
	    Marshal.WriteIntPtr(methodDef, (3 * IntPtr.Size), IntPtr.Zero);
	    pyHandle = Runtime.PyCFunction_New(methodDef, IntPtr.Zero);

	    keepAlive.Add(this);
	}

	public IntPtr Handle {
	    get { return pyHandle; }
	}

	public IntPtr Call(IntPtr args, IntPtr kw) {
	    return Runtime.PyCFunction_Call(pyHandle, args, kw);
	}

    }


}

