// Copyright (c) 2001, 2002 Zope Corporation and Contributors.
//
// All Rights Reserved.
//
// This software is subject to the provisions of the Zope Public License,
// Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
// WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
// FOR A PARTICULAR PURPOSE.

using System;
using System.Runtime.InteropServices;

namespace Python.Runtime {


    public class PyDict : PyObject {

	public PyDict(IntPtr pointer) : base(pointer) {}

	public PyDict() : base(IntPtr.Zero) {
	    handle = Runtime.PyDict_New();
	    if (handle == IntPtr.Zero) {
		throw new PythonException();
	    }
	}

	public override int Length {
	    get {
		return Runtime.PyDict_Size(handle);
	    }
	}


	public override PyObject GetItem(PyObject key) {
	    IntPtr item = Runtime.PyDict_GetItem(handle, key.handle);
	    if (item == IntPtr.Zero) {
		throw new PythonException();
	    }
	    Runtime.Incref(item);
	    return new PyObject(item);
	}

	public override void SetItem(PyObject key, PyObject item) {
	    int n = Runtime.PyDict_SetItem(handle, key.handle, item.handle);
	    if (n < 0) {
		throw new PythonException();
	    }
	}

	public override void DelItem(PyObject key) {
	    int n = Runtime.PyDict_DelItem(handle, key.handle);
	    if (n < 0) {
		throw new PythonException();
	    }
	}


	public bool HasKey(PyObject key) {
	    return (Runtime.PyMapping_HasKey(handle, key.handle) != 0);
	}

	public bool HasKey(string key) {
	    return HasKey(new PyString(key));
	}

	public PyObject Keys() {
	    IntPtr items = Runtime.PyDict_Keys(handle);
	    if (items == IntPtr.Zero) {
		throw new PythonException();
	    }
	    return new PyObject(items);
	}

	public PyObject Values() {
	    IntPtr items = Runtime.PyDict_Values(handle);
	    if (items == IntPtr.Zero) {
		throw new PythonException();
	    }
	    return new PyObject(items);
	}

	public PyObject Items() {
	    IntPtr items = Runtime.PyDict_Items(handle);
	    if (items == IntPtr.Zero) {
		throw new PythonException();
	    }
	    return new PyObject(items);
	}

	public PyDict Copy() {
	    IntPtr copy = Runtime.PyDict_Copy(handle);
	    if (copy == IntPtr.Zero) {
		throw new PythonException();
	    }
	    return new PyDict(copy);
	}

	public void Update(PyObject other) {
	    int result = Runtime.PyDict_Update(handle, other.handle);
	    if (result < 0) {
		throw new PythonException();
	    }
	}

	public void Clear() {
	    Runtime.PyDict_Clear(handle);
	}

    }


}
