// Copyright (c) 2001, 2002 Zope Corporation and Contributors.
//
// All Rights Reserved.
//
// This software is subject to the provisions of the Zope Public License,
// Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
// WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
// FOR A PARTICULAR PURPOSE.

using System;
using System.Runtime.InteropServices;

namespace Python.Runtime {


    public class PyInteger : PyNumber {

	private bool cached = false;
	private int ival = 0;

	public PyInteger(IntPtr pointer) : base(pointer) {}

	public PyInteger(string value) : base(IntPtr.Zero) {
	    handle = Runtime.PyInt_FromString(value, IntPtr.Zero, 0);
	    if (handle == IntPtr.Zero) {
		throw new PythonException();
	    }
	}

	public PyInteger(int value) : base(IntPtr.Zero) {
	    handle = Runtime.PyInt_FromLong(value);
	    if (handle == IntPtr.Zero) {
		throw new PythonException();
	    }
	    ival = value;
	    cached = true;
	}

	public PyInteger(long value) : base(IntPtr.Zero) {
	    if (value > Int32.MaxValue || value < Int32.MinValue) {
		throw new OverflowException("Value too large");
	    }
	    handle = Runtime.PyInt_FromLong((int)value);
	    if (handle == IntPtr.Zero) {
		throw new PythonException();
	    }
	    ival = (int)value;
	    cached = true;
	}

	public PyInteger(bool value) : this((int)(value ? 1 : 0)) {}

	public PyInteger(short value) : this((int)value) {}



	public int Value {
	    get {
		if (!cached) {
		    ival = Runtime.PyInt_AsLong(handle);
		}
		return ival;
	    }
	}


	public bool ToBoolean() {
	    return (this.Value != 0);
	}

	public short ToInt16() {
	    return System.Convert.ToInt16(this.Value);
	}

	public int ToInt32() {
	    return this.Value;
	}

	public long ToInt64() {
	    return System.Convert.ToInt64(this.Value);
	}

	public double ToDouble() {
	    return System.Convert.ToDouble(this.Value);
	}

    }


}
