// Copyright (c) 2001, 2002 Zope Corporation and Contributors.
//
// All Rights Reserved.
//
// This software is subject to the provisions of the Zope Public License,
// Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
// WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
// FOR A PARTICULAR PURPOSE.

using System;

namespace Python.Runtime {


    public class PyList : PySequence {

	public PyList(IntPtr pointer) : base(pointer) {}

	public PyList() : base(IntPtr.Zero) {
	    handle = Runtime.PyList_New(0);
	    if (handle == IntPtr.Zero) {
		throw new PythonException();
	    }
	}

	public PyList(PyObject[] items) : base(IntPtr.Zero) {
	    int size = items.Length;
	    int result;

	    handle = Runtime.PyList_New(size);
	    if (handle == IntPtr.Zero) {
		throw new PythonException();
	    }
	    for (int i = 0; i < size; i++) {
		Runtime.Incref(items[i].handle);
		result = Runtime.PyList_SetItem(handle, i, items[i].handle);
		if (result < 0) {
		    throw new PythonException();
		}
	    }
	}


	public void Append(PyObject item) {
	    int result = Runtime.PyList_Append(handle, item.Handle);
	    if (result < 0) {
		throw new PythonException();
	    }
	}

	public void Insert(int index, PyObject item) {
	    int result = Runtime.PyList_Insert(handle, index, item.Handle);
	    if (result < 0) {
		throw new PythonException();
	    }
	}

	public void Reverse() {
	    int result = Runtime.PyList_Reverse(handle);
	    if (result < 0) {
		throw new PythonException();
	    }
	}

	public void Sort() {
	    int result = Runtime.PyList_Sort(handle);
	    if (result < 0) {
		throw new PythonException();
	    }
	}

    }


}
