// Copyright (c) 2001, 2002 Zope Corporation and Contributors.
//
// All Rights Reserved.
//
// This software is subject to the provisions of the Zope Public License,
// Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
// WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
// FOR A PARTICULAR PURPOSE.

using System;
using System.Runtime.InteropServices;

namespace Python.Runtime {


    public class PySequence : PyObject {

	public PySequence(IntPtr pointer) : base(pointer) {
	}

	public override int Length {
	    get {
		return Runtime.PySequence_Size(handle);
	    }
	}

	public override PyObject GetItem(int index) {
	    IntPtr item = Runtime.PySequence_GetItem(handle, index);
	    if (item == IntPtr.Zero) {
		throw new PythonException();
	    }
	    return new PyObject(item);
	}

	public override void SetItem(int index, PyObject item) {
	    int result = Runtime.PySequence_SetItem(handle, index, item.Handle);
	    if (result < 0) {
		throw new PythonException();
	    }
	}

	public override void DelItem(int index) {
	    int result = Runtime.PySequence_DelItem(handle, index);
	    if (result < 0) {
		throw new PythonException();
	    }
	}


	// add slice ops
	// add op overloads

	public int Index(PyObject item) {
	    return Runtime.PySequence_Index(handle, item.handle);
	}

	public int Count(PyObject value) {
	    int result = Runtime.PySequence_Count(handle, value.handle);
	    if (result < 0) {
		Runtime.PyErr_Clear();
		return 0;
	    }
	    return result;
	}

	public bool Contains(PyObject other) {
	    int result = Runtime.PySequence_Contains(handle, other.handle);
	    if (result < 0) {
		Runtime.PyErr_Clear();
		return false;
	    }
	    return (result != 0);
	}

	public PyObject Concat(PyObject other) {
	    IntPtr result = Runtime.PySequence_Concat(handle, other.handle);
	    if (result == IntPtr.Zero) {
		throw new PythonException();
	    }
	    return new PyObject(result);
	}

	public PyObject Repeat(int count) {
	    IntPtr result = Runtime.PySequence_Repeat(handle, count);
	    if (result == IntPtr.Zero) {
		throw new PythonException();
	    }
	    return new PyObject(result);
	}

	public static PyTuple AsTuple(PyObject value) {
	    IntPtr result = Runtime.PySequence_Tuple(value.handle);
	    if (result == IntPtr.Zero) {
		throw new PythonException();
	    }
	    return new PyTuple(result);
	}

	public static PyList AsList(PyObject value) {
	    IntPtr result = Runtime.PySequence_List(value.handle);
	    if (result == IntPtr.Zero) {
		throw new PythonException();
	    }
	    return new PyList(result);
	}


    }


}
