// Copyright (c) 2001, 2002 Zope Corporation and Contributors.
//
// All Rights Reserved.
//
// This software is subject to the provisions of the Zope Public License,
// Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
// WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
// FOR A PARTICULAR PURPOSE.

using System;
using System.Collections;

namespace Python.Runtime {


    public class PythonInterpreter {

	public static bool initialized;

	//private static IntPtr mainThread;
	//private static Hashtable threads;

	public static void Initialize() {
	    if (!initialized) {
		Runtime.Initialize();
		initialized = true;
	    }
	}


	public static void TestInit() {
	    /*

	      First time decref on a gc-aware object:
	        PyObject_GC_UnTrack(ob)


	     */
	    Runtime.Py_Initialize();
	    initialized = true;

	    // always causes segfault
	    IntPtr d = Runtime.PyDict_New();
	    Runtime.Decref(d);

	    IntPtr t = Runtime.PyTuple_New(0);
	    Runtime.Decref(t);

	    // this always works (non-gc aware)
	    IntPtr i = Runtime.PyInt_FromLong(550);
	    Runtime.Decref(i);

	    IntPtr l = Runtime.PyLong_FromLong(550);
	    Runtime.Decref(l);

    }



	public static void Finalize() {
	    if (initialized) {
		Runtime.Py_Finalize();
		initialized = false;
	    }
	}


	public static bool IsInitialized {
	    get {
		return initialized;
	    }
	}

	public static string ProgramName {
	    get {
		string result = Runtime.Py_GetProgramName();
		if (result == null) {
		    return "";
		}
		return result;
	    }
	    set {
		Runtime.Py_SetProgramName(value);
	    }
	}

	public static string PythonHome {
	    get {
		string result = Runtime.Py_GetPythonHome();
		if (result == null) {
		    return "";
		}
		return result;
	    }
	    set {
		Runtime.Py_SetPythonHome(value);
	    }
	}

	public static string Version {
	    get { 
		return Runtime.Py_GetVersion(); 
	    }
	}

	public static string Platform {
	    get { 
		return Runtime.Py_GetPlatform(); 
	    }
	}

	public static string Copyright {
	    get { 
		return Runtime.Py_GetCopyright(); 
	    }
	}

	public static string Compiler {
	    get { 
		return Runtime.Py_GetCompiler(); 
	    }
	}

	public static string BuildInfo {
	    get { 
		return Runtime.Py_GetBuildInfo(); 
	    }
	}


	// TODO: support InitThreads, etc.

	/*
	  public void InitThreads() {}

	  public static void RegisterThread() {
	  IntPtr threadState = threads.Get(Thread.CurrentThread);
	  if (threadState != null) {
	  return;
	  }
	  Runtime.PyEval_AcquireLock();
	
	  }

	  public static void FinalizeThread() {
	  IntPtr threadState = threads.Get(Thread.CurrentThread);
	  if (threadState == null) {
	  throw
	  }
	  Runtime.PyEval_AcquireLock();
	  Runtime.PyThreadState_Swap(IntPtr.Zero);
	  Runtime.PyThreadState_Clear(threadState);
	  Runtime.PyThreadState_Delete(threadState);
	  // remove it!
	  Runtime.PyEval_ReleaseLock();
	  }

	*/


	public static int RunSimpleString(string code) {
	    return Runtime.PyRun_SimpleString(code);
	}

	public static PyModule ImportModule(string name) {
	    PyString moduleName = new PyString(name);
	    IntPtr pointer = Runtime.PyImport_Import(moduleName.Handle);
	    return new PyModule(pointer);
	}

	public static PyModule ReloadModule(PyModule module) {
	    IntPtr pointer = Runtime.PyImport_ReloadModule(module.Handle);
	    return new PyModule(pointer);
	}

	public static PyModule AddModule(string name) {
	    IntPtr result = Runtime.PyImport_AddModule(name);
	    if (result == IntPtr.Zero) {
		throw new PythonException();
	    }
	    Runtime.Incref(result);
	    return new PyModule(result);
	}

    }


}
