// Copyright (c) 2001, 2002 Zope Corporation and Contributors.
//
// All Rights Reserved.
//
// This software is subject to the provisions of the Zope Public License,
// Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
// THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
// WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
// WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
// FOR A PARTICULAR PURPOSE.

using System;
using System.Runtime.InteropServices;
using System.Reflection.Emit;
using System.Collections;
using System.Reflection;


using System.Threading;

namespace Python.Runtime {

    //=======================================================================
    // The TypeManager class is responsible for building binary-compatible
    // Python type objects that are connected to a managed implementation.
    //=======================================================================

    internal class TypeManager {

	static ArrayList dList;
	static Hashtable cache;
	static IntPtr p_thunk;
	static int obSize;
	static int tpSize;

	static TypeManager() {
	    obSize = Marshal.SizeOf(new PyObjectHead());
	    tpSize = Marshal.SizeOf(new PyTypeObject());
	    p_thunk = Marshal.AllocHGlobal(IntPtr.Size);
	    cache = new Hashtable();
	    dList = new ArrayList();
	    a = new ArrayList();
	}


	internal static IntPtr GetTypeHandle(Type obType) {
	    Object ob = cache[obType];
	    if (ob != null) {
		return (IntPtr) ob;
	    }
	    IntPtr tp = CreateType(obType);
	    cache[obType] = tp;
	    return tp;
	}

	internal static IntPtr GetTypeHandle(ManagedType obj, Type clrType) {
	    Object ob = cache[clrType];
	    if (ob != null) {
		return (IntPtr) ob;
	    }
	    IntPtr tp = CreateType(obj, clrType);
	    cache[clrType] = tp;
	    return tp;
	}


	static AssemblyBuilder ab;
	static ModuleBuilder mb;
	static ArrayList a;

	internal static IntPtr GetThunk2(MethodInfo method) {
	    // this doesn't work because SigHelper expects to work with
	    // a generated module. If we could find a way to get a sig
	    // and length from a methodinfo, we could really make this
	    // simple...
	    if (mb == null) {
		AssemblyName aname = new AssemblyName();
		aname.Name = "__";
		AssemblyBuilderAccess aa = AssemblyBuilderAccess.Run;
		
		ab = Thread.GetDomain().DefineDynamicAssembly(aname, aa);
		mb = ab.DefineDynamicModule("__");
	    }

	    SignatureHelper s = SignatureHelper.GetMethodSigHelper(
								   //mb, //
								   method.DeclaringType.Module,
				  CallingConvention.Cdecl,
				  method.ReturnType
				  );

	    ParameterInfo[] pi = method.GetParameters();
	    for (int n = 0; n < pi.Length; n++) {
		s.AddArgument(pi[n].ParameterType);
	    }
	    byte[] bsig = s.GetSignature();
	    int csig = bsig.Length;
	    
	    IntPtr psig = Marshal.AllocHGlobal(bsig.Length);
	    Marshal.Copy(bsig, 0, psig, csig);

	    IntPtr fp = Marshal.GetUnmanagedThunkForManagedMethodPtr(
				method.MethodHandle.GetFunctionPointer(), 
				psig, 
				csig
				);
	    //Marshal.FreeHGlobal(psig);

	    a.Add(fp);
	    a.Add(s);
	    return fp;

	}





	private static IntPtr GetThunk(MethodInfo method) {
	    Type dt = typeof(CallbackThunks).GetNestedType(method.Name);
	    //Type dt = Prototypes.GetPrototype(method.Name);
	    if (dt != null) {
		Delegate d = Delegate.CreateDelegate(dt, method);
		CallbackThunk cb = new CallbackThunk(d);
		Marshal.StructureToPtr(cb, p_thunk, false);
		IntPtr fp = Marshal.ReadIntPtr(p_thunk);
		dList.Add(d);
		return fp;
	    }
	    return IntPtr.Zero;
	}


	// Given a PyTypeObject instance, fill the slot pointer members of
	// the PyTypeObject with thunks pointing to methods defined by the
	// given implementing type.

	private static void InitializeSlots(PyTypeObject typeObj, Type type) {
	    BindingFlags flags = BindingFlags.Public | BindingFlags.Static;
	    Type typedesc = typeof(PyTypeObject);

	    while (type != null) {
		MethodInfo[] methods = type.GetMethods(flags);
		for (int i = 0; i < methods.Length; i++) {
		    MethodInfo method = methods[i];

		    string name = method.Name;
		    if (! (name.StartsWith("tp_") ||
			   name.StartsWith("nb_") ||
			   name.StartsWith("sq_") ||
			   name.StartsWith("bf_")
			   ) ) {
			continue;
		    }

		    FieldInfo fi = typedesc.GetField(name);
		    IntPtr value = (IntPtr) fi.GetValue(typeObj);
		    if ((fi != null) && (value == IntPtr.Zero)) {
			fi.SetValue(typeObj, GetThunk(method));
		    }
		}
		type = type.BaseType;
	    }
	}


	// Given an initialized PyTypeObject, add type methods (implemented
	// by the passed in managed type) to the type dictionary.

	private static void InitMethods(IntPtr pytype, Type type) {
	    IntPtr ppdict = Runtime._PyObject_GetDictPtr(pytype);
	    IntPtr dict = Marshal.ReadIntPtr(ppdict);
	    Type marker = typeof(PythonMethodAttribute);

	    BindingFlags flags = BindingFlags.Public | BindingFlags.Static;

	    while (type != null) {
		MethodInfo[] methods = type.GetMethods(flags);
		for (int i = 0; i < methods.Length; i++) {
		    MethodInfo method = methods[i];
		    object[] attrs = method.GetCustomAttributes(marker, false);
		    if (attrs.Length > 0) {
			string method_name = method.Name;
			MethodInfo[] mi = new MethodInfo[1];
			mi[0] = method;
			MethodObject m = new TypeMethod(method_name, mi);
			Runtime.PyDict_SetItemString(dict, method_name,
						     m.Handle);
		    }
		}
		type = type.BaseType;
	    }
	}


	// fixup internal offsets to protocol method structs.

	public static void FixupOffsets(IntPtr op) {
	    IntPtr nboffset = new IntPtr(op.ToInt32() + (47 * IntPtr.Size));
	    Marshal.WriteIntPtr(op, (12 * IntPtr.Size), nboffset);

	    IntPtr sqoffset = new IntPtr(op.ToInt32() + (85 * IntPtr.Size));
	    Marshal.WriteIntPtr(op, (13 * IntPtr.Size), sqoffset);

	    IntPtr mpoffset = new IntPtr(op.ToInt32() + (95 * IntPtr.Size));
	    Marshal.WriteIntPtr(op, (14 * IntPtr.Size), mpoffset);

	    IntPtr bfoffset = new IntPtr(op.ToInt32() + (98 * IntPtr.Size));
	    Marshal.WriteIntPtr(op, (20 * IntPtr.Size), bfoffset);
	}


	// Now blit the PyTypeObject instance to the unmanaged heap. This
	// bit lets us avoid pinning objects, which would disrupt the GC.

	private static IntPtr FinishType(PyTypeObject t) {
	    IntPtr op = Runtime.PyMem_Malloc(tpSize);
	    GCHandle gch = GCHandle.Alloc(t, GCHandleType.Pinned);
	    Marshal.StructureToPtr(t, op, false);

	    FixupOffsets(op);

	    Runtime.PyType_Ready(op);
	    gch.Free();
	    return op;
	}



	// Initialize a new binary-compatible PyTypeObject for a simple
	// extension type. A separate overload handles reflected types.

	internal static IntPtr CreateType(Type obType) {

	    PyTypeObject pyTypeObj = new PyTypeObject();
	    pyTypeObj.tp_name = Marshal.StringToHGlobalAnsi(obType.Name);
	    pyTypeObj.ob_type = Runtime.PyTypeType;

	    pyTypeObj.tp_basicsize = (IntPtr)(3 * IntPtr.Size);
	    pyTypeObj.ob_refcnt = (IntPtr) 1;  // ??
	    pyTypeObj.tp_flags = (IntPtr) (PyTypeFlags.Default | 
					   PyTypeFlags.Managed);

	    InitializeSlots(pyTypeObj, obType);

	    IntPtr o = FinishType(pyTypeObj);
	    InitMethods(o, obType);
	    return o;
	}


	// Initialize a new binary-compatible PyTypeObject for a Python
	// type that reflects a managed type.

	internal static IntPtr CreateType(ManagedType obj, Type clrType) {
	    // Cleanup the type name to get rid of funny nested type names.
	    string name = "CLR." + clrType.FullName;
	    int i = name.LastIndexOf('+');
	    if (i > -1) {
		name = name.Substring(i + 1);
	    }

	    // todo - can blit this after first time!
	    PyTypeObject pyTypeObj = new PyTypeObject();
	    pyTypeObj.tp_name = Marshal.StringToHGlobalAnsi(name);

	    pyTypeObj.ob_type = Runtime.PyCLRMetaType;
	    obj.tpHandle = pyTypeObj.ob_type;
	    obj.gcHandle = GCHandle.Alloc(obj);

	    pyTypeObj.ob_size = (IntPtr)obj.gcHandle;
	    pyTypeObj.tp_basicsize = (IntPtr)(3 * IntPtr.Size);
	    pyTypeObj.ob_refcnt = (IntPtr) 1;  // ??
	    pyTypeObj.tp_flags = (IntPtr) (PyTypeFlags.Default | 
					   PyTypeFlags.BaseType |
					   PyTypeFlags.HeapType |
					   PyTypeFlags.Managed);
	    //pyTypeObj.tp_dictoffset = (IntPtr) (3 * IntPtr.Size);

	    InitializeSlots(pyTypeObj, obj.GetType());

	    IntPtr op = FinishType(pyTypeObj);
	    obj.pyHandle = op;
	    return op;
	}


    }






}
