// PyIntegerTests.cs - Unit tests for Python dict wrappers
//
// Author:  Brian Lloyd <brian@zope.com>
//
// (c) 2002 Brian Lloyd

using NUnit.Framework;
using Python.Runtime;
using System;

namespace Python.Runtime.Tests {


public class PyIntegerTests : TestCase {

    public PyIntegerTests(string name) : base(name) {}
    
    public static ITest Suite {
	get {
	    return new TestSuite(typeof(PyIntegerTests));
	}
    }

    protected override void SetUp() {
	PythonInterpreter.Initialize();
    }

    protected override void TearDown() {	
	PythonInterpreter.Finalize();
    }

    // --------------------------------------------------------------------
    // Test cases
    // --------------------------------------------------------------------


    public void TestPyIntegerConstructors() {
	PyInteger pyobj = new PyInteger(42);
	Assert("Handle 1", pyobj.Handle != IntPtr.Zero);
	Assert("Refcount 1", pyobj.RefCount > 1);
	Assert("Value 1", pyobj.ToInt32() == 42);

	int ivalue = 42;
	pyobj = new PyInteger(ivalue);
	Assert("Handle 2", pyobj.Handle != IntPtr.Zero);
	Assert("Refcount 2", pyobj.RefCount > 1);
	Assert("Value 2", pyobj.ToInt32() == 42);

	long lvalue = 42;
	pyobj = new PyInteger(lvalue);
	Assert("Handle 3", pyobj.Handle != IntPtr.Zero);
	Assert("Refcount 3", pyobj.RefCount > 1);
	Assert("Value 3", pyobj.ToInt32() == 42);

	pyobj = new PyInteger(true);
	Assert("Handle 4", pyobj.Handle != IntPtr.Zero);
	Assert("Refcount 4", pyobj.RefCount > 1);
	Assert("Value 4", pyobj.ToInt32() == 1);

	pyobj = new PyInteger(false);
	Assert("Handle 5", pyobj.Handle != IntPtr.Zero);
	Assert("Refcount 5", pyobj.RefCount > 1);
	Assert("Value 5", pyobj.ToInt32() == 0);
	
	short svalue = 42;
	pyobj = new PyInteger(svalue);
	Assert("Handle 6", pyobj.Handle != IntPtr.Zero);
	Assert("Refcount 6", pyobj.RefCount > 1);
	Assert("Value 6", pyobj.ToInt32() == 42);
	
	pyobj = new PyInteger("42");
	Assert("Handle 7", pyobj.Handle != IntPtr.Zero);
	Assert("Refcount 7", pyobj.RefCount > 1);
	Assert("Value 7", pyobj.ToInt32() == 42);

	PyObject module = PythonInterpreter.ImportModule("sys");
	PyInteger result = (PyInteger)module.InvokeMethod(
			              "getrefcount", pyobj
				      );
	Assert("Handle 8", result.Handle != IntPtr.Zero);
	Assert("Refcount 8", result.RefCount > 1);
	Assert("Value 8", result.ToInt32() > 0);

	PyObject output = module.InvokeMethod("exc_info");
	Console.WriteLine("result was: {0}", output);

	try {
	    pyobj = new PyInteger(((long)(Int32.MaxValue) + 1));
	    Fail("Expected OverflowError 1");
	}
	catch (OverflowException) {
	}

	try {
	    pyobj = new PyInteger(((long)(Int32.MinValue) - 1));
	    Fail("Expected OverflowError 2");
	}
	catch (OverflowException) {
	}

    }

    public void TestPyIntegerDispose() {
	PyInteger pyobj = new PyInteger(42);
	Assert("Handle 1", pyobj.Handle != IntPtr.Zero);
	Assert("Refcount 1", pyobj.RefCount > 1);
	pyobj.Dispose();
	Assert("Handle 2", pyobj.Handle == IntPtr.Zero);
	Assert("Refcount 2", pyobj.RefCount == 0);
    }

    public void TestPyIntegerToBoolean() {
	PyInteger pyobj = new PyInteger(1);
	Assert("Value 1", pyobj.ToBoolean() == true);

	pyobj = new PyInteger(-1);
	Assert("Value 2", pyobj.ToBoolean() == true);

	pyobj = new PyInteger(0);
	Assert("Value 3", pyobj.ToBoolean() == false);
    }

    public void TestPyIntegerToInt16() {
	PyInteger pyobj = new PyInteger(42);
	Assert("Value 1", pyobj.ToInt16() == 42);

	pyobj = new PyInteger(Int16.MaxValue);
	Assert("Value 2", pyobj.ToInt16() == Int16.MaxValue);

	pyobj = new PyInteger(Int16.MinValue);
	Assert("Value 3", pyobj.ToInt16() == Int16.MinValue);

	short sval;

	try {
	    pyobj = new PyInteger(Int16.MaxValue + 1);
	    sval = pyobj.ToInt16();
	    Fail("Expected OverflowError 1");
	}
	catch (OverflowException) {
	}

	try {
	    pyobj = new PyInteger(Int16.MinValue - 1);
	    sval = pyobj.ToInt16();
	    Fail("Expected OverflowError 2");
	}
	catch (OverflowException) {
	}
    }

    public void TestPyIntegerToInt32() {
	PyInteger pyobj = new PyInteger(42);
	Assert("Value 1", pyobj.ToInt32() == 42);

	pyobj = new PyInteger(Int32.MaxValue);
	Assert("Value 2", pyobj.ToInt32() == Int32.MaxValue);

	pyobj = new PyInteger(Int32.MinValue);
	Assert("Value 3", pyobj.ToInt32() == Int32.MinValue);


    }

    public void TestPyIntegerToInt64() {
	PyInteger pyobj = new PyInteger(42);
	Assert("Value 1", pyobj.ToInt64() == 42);

	pyobj = new PyInteger(Int32.MaxValue);
	Assert("Value 2", pyobj.ToInt64() == Int32.MaxValue);

	pyobj = new PyInteger(Int32.MinValue);
	Assert("Value 3", pyobj.ToInt64() == Int32.MinValue);
    }

    public void TestPyIntegerToDouble() {
	PyInteger pyobj = new PyInteger(42);
	Assert("Value 1", pyobj.ToDouble() == 42);

	pyobj = new PyInteger(Int32.MaxValue);
	Assert("Value 2", pyobj.ToDouble() == Int32.MaxValue);

	pyobj = new PyInteger(Int32.MinValue);
	Assert("Value 3", pyobj.ToDouble() == Int32.MinValue);
    }


}


} // Python.Runtime.Tests
