// PyModuleTests.cs - Unit tests for the Python.Runtime assembly
//
// Author:  Brian Lloyd <brian@zope.com>
//
// (c) 2002 Brian Lloyd

using NUnit.Framework;
using Python.Runtime;
using System;

namespace Python.Runtime.Tests {


public class PyModuleTests : TestCase {

    public PyModuleTests(string name) : base(name) {}
    
    public static ITest Suite {
	get {
	    return new TestSuite(typeof(PyModuleTests));
	}
    }

    protected override void SetUp() {
	PythonInterpreter.Initialize();
    }

    protected override void TearDown() {	
	PythonInterpreter.Finalize();
    }

    // Actual test cases start here.

    public void TestModuleInterfaces() {
	PyModule module = PythonInterpreter.ImportModule("rfc822");
	Assert(module.Handle != IntPtr.Zero);
	Assert(module.Name == "rfc822");

	string filename = module.Filename;
	Assert(filename.EndsWith("rfc822.py") ||
	       filename.EndsWith("rfc822.pyc") || 
	       filename.EndsWith("rfc822.pyo"));

	Assert(module.HasAttr("Message") == true);

	PyObject message = module.GetAttr("Message");
	Assert(message.Handle != IntPtr.Zero);


	Assert(module.ToString().StartsWith("<module 'rfc822'"));

	Assert(module.Repr().StartsWith("<module 'rfc822'"));

	Assert( (IntPtr)0 == IntPtr.Zero );
    }
    

}


} // Python.Runtime.Tests
