// PyStringTests.cs - Unit tests for the Python.Runtime assembly
//
// Author:  Brian Lloyd <brian@zope.com>
//
// (c) 2002 Brian Lloyd

using NUnit.Framework;
using Python.Runtime;
using System;

namespace Python.Runtime.Tests {


public class PyStringTests : TestCase {

    public PyStringTests(string name) : base(name) {}
    
    public static ITest Suite {
	get {
	    return new TestSuite(typeof(PyStringTests));
	}
    }

    protected override void SetUp() {
	PythonInterpreter.Initialize();
    }

    protected override void TearDown() {	
	PythonInterpreter.Finalize();
    }

    // Actual test cases start here.

    public void TestObjectCreation() {
	PyString theString = new PyString("Bring me... a shrubbery!");
	Assert(theString.ToString() == "Bring me... a shrubbery!");
	Assert(theString.Repr() == "'Bring me... a shrubbery!'");
	Assert(theString.Length == 24);
    }

    public void TestObjectRefCount() {
	PyString value = new PyString("This is a test!");
	Assert(value.RefCount == 1);
	Assert(value.Handle != IntPtr.Zero);

	value.Dispose();
	Assert(value.RefCount == 0);
	Assert(value.Handle == IntPtr.Zero);
    }
  
    public void TestObjectBasics() {
	PyString theString = new PyString("This is a test!");
	Assert(theString.Handle != IntPtr.Zero);

	// fix this
	PyObject theType = theString.PythonType;
	//	Console.WriteLine("Python type is: {0}", theType.Repr());


	Assert(theString.HasAttr(new PyString("lower")) == true);
	Assert(theString.HasAttr("lower") == true);
	
	PyObject method = theString.GetAttr(new PyString("lower"));
	Assert(method.Handle != IntPtr.Zero);

	method = theString.GetAttr("lower");
	Assert(method.Handle != IntPtr.Zero);

	// Setattr tests should fail!
	//	module.SetAttr(new PyString("SetAttrTest"), method);
	//	method = module.GetAttr("SetAttrTest");
	//	Assert(method.Handle != 0);
	//
	//	module.SetAttr("SetAttrTest", method);
	//	method = module.GetAttr("SetAttrTest");
	//	Assert(method.Handle != 0);

	Assert(theString.IsCallable() == false);
	Assert(theString.IsTrue() == true);
	Assert(theString.Length == 15);

	Assert(theString.ToString() == "This is a test!");
	Assert(theString.Repr() == "'This is a test!'");
    }

}


} // Python.Runtime.Tests
