# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
#
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.

from CLR.Python.Test import ClassTest
import sys, os, string, unittest, types
import CLR.System as System

class ClassTests(unittest.TestCase):
    """Test CLR class support."""

    def testBasicReferenceType(self):
        """Test usage of CLR defined reference types."""
        String = System.String
        self.assertEquals(String.Empty, "")

    def testBasicValueType(self):
        """Test usage of CLR defined value types."""
        Int32 = System.Int32
        self.assertEquals(Int32.MaxValue, 2147483647)

    def testClassStandardAttrs(self):
        """Test standard class attributes."""
        self.failUnless(ClassTest.__name__ == 'ClassTest')
        self.failUnless(ClassTest.__module__ == 'CLR.Python.Test')
        self.failUnless(type(ClassTest.__dict__) == types.DictProxyType)
        self.failUnless(ClassTest.__doc__ == None)

    def testNonPublicClass(self):
        """Test that non-public classes are inaccessible."""
        from CLR.Python import Test

        def test():
            from CLR.Python.Test import InternalClass

        self.failUnlessRaises(ImportError, test)

        def test():
            x = Test.InternalClass

        self.failUnlessRaises(AttributeError, test)


def test_suite():
    return unittest.makeSuite(ClassTests)

def main():
    unittest.TextTestRunner().run(test_suite())

if __name__ == '__main__':
    testcase.setup()
    main()

