# Copyright (c) 2001, 2002 Zope Corporation and Contributors.
#
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.

import sys, os, string, unittest, types
from CLR.Python.Test import MethodTest

class MethodTests(unittest.TestCase):
    """Test CLR method support."""

    def testInstanceMethodDescriptor(self):
        """Test instance method descriptor behavior."""
        def test():
            MethodTest().PublicMethod = 0

        self.failUnlessRaises(AttributeError, test)

        def test():
            MethodTest.PublicMethod = 0

        self.failUnlessRaises(AttributeError, test)

        def test():
            del MethodTest().PublicMethod

        self.failUnlessRaises(AttributeError, test)

        def test():
            del MethodTest.PublicMethod

        self.failUnlessRaises(AttributeError, test)

    def testStaticMethodDescriptor(self):
        """Test static method descriptor behavior."""
        def test():
            MethodTest().PublicStaticMethod = 0

        self.failUnlessRaises(AttributeError, test)

        def test():
            MethodTest.PublicStaticMethod = 0

        self.failUnlessRaises(AttributeError, test)

        def test():
            del MethodTest().PublicStaticMethod

        self.failUnlessRaises(AttributeError, test)

        def test():
            del MethodTest.PublicStaticMethod

        self.failUnlessRaises(AttributeError, test)

    def testPublicInstanceMethod(self):
        """Test public instance method visibility."""
        object = MethodTest();
        self.failUnless(object.PublicMethod() == "public")

    def testPublicStaticMethod(self):
        """Test public static method visibility."""
        object = MethodTest();
        self.failUnless(MethodTest.PublicStaticMethod() == "public static")
        self.failUnless(object.PublicStaticMethod() == "public static")

    def testProtectedInstanceMethod(self):
        """Test protected instance method visibility."""
        object = MethodTest();
        self.failUnless(object.ProtectedMethod() == "protected")

    def testProtectedStaticMethod(self):
        """Test protected static method visibility."""
        object = MethodTest();
        result = "protected static"
        self.failUnless(MethodTest.ProtectedStaticMethod() == result)
        self.failUnless(object.ProtectedStaticMethod() == result)

    def testInternalMethod(self):
        """Test internal method visibility."""
        def test():
            f = MethodTest().InternalMethod

        self.failUnlessRaises(AttributeError, test)

        def test():
            f = MethodTest.InternalMethod

        self.failUnlessRaises(AttributeError, test)

        def test():
            f = MethodTest().InternalStaticMethod

        self.failUnlessRaises(AttributeError, test)

        def test():
            f = MethodTest.InternalStaticMethod

        self.failUnlessRaises(AttributeError, test)

    def testPrivateMethod(self):
        """Test private method visibility."""
        def test():
            f = MethodTest().PrivateMethod

        self.failUnlessRaises(AttributeError, test)

        def test():
            f = MethodTest.PrivateMethod

        self.failUnlessRaises(AttributeError, test)

        def test():
            f = MethodTest().PrivateStaticMethod

        self.failUnlessRaises(AttributeError, test)

        def test():
            f = MethodTest.PrivateStaticMethod

        self.failUnlessRaises(AttributeError, test)

    def testMethodDescriptorAbuse(self):
        """Test method descriptor abuse."""
        desc = MethodTest.__dict__['PublicMethod']

        def test():
            desc.__get__(0, 0)

        self.failUnlessRaises(TypeError, test)

        def test():
            desc.__set__(0, 0)

        self.failUnlessRaises(AttributeError, test)



def test_suite():
    return unittest.makeSuite(MethodTests)

def main():
    unittest.TextTestRunner().run(test_suite())

if __name__ == '__main__':
    testcase.setup()
    main()

